# -*- coding: utf-8 -*-

# Description: Remark.TagParsers package initialization
# Documentation: tag_parsers.txt

# Remark 1.6.0
# Copyright (c) 2009 - 2013 
# Kalle Rutanen
# Distributed under the MIT license (see license.txt).

