# Description: Version-specific constants
# Documentation: implementation.txt

# Remark 1.6.0
# Copyright (c) 2009 - 2013 
# Kalle Rutanen
# Distributed under the MIT license (see license.txt).

def remarkVersion():
    '''
    Returns the version of Remark as a string.
    '''
    return '1.6.0'

def asciiMathMlName():
    '''
    Returns the name of the Javascript file to
    use for AsciiMathML.
    '''
    return 'ASCIIMathMLwFallback.js'
